@@ -106,10 +106,9 @@ def prevent_zombie(method):
     def wrapper(self, pid, *args, **kwargs):
         try:
             return method(self, pid, *args, **kwargs)
-        except IOError, err:
+        except (OSError, IOError), err:
             if err.errno == errno.ENOENT:  # no such file or directory
-                if not self.pid_exists(pid):
-                    raise NoSuchProcess(pid)
+                raise NoSuchProcess(pid)
             raise
     return wrapper
 
@@ -122,8 +121,8 @@ def wrap_privileges(callable):
     def wrapper(*args, **kwargs):
         try:
             return callable(*args, **kwargs)
-        except OSError, err:
-            if err.errno == errno.EPERM:
+        except (OSError, IOError), err:
+            if err.errno in (errno.EPERM, errno.EACCES):
                 raise AccessDenied
             raise
     return wrapper
@@ -254,6 +253,31 @@ class Impl(object):
         f.close()
         return (resident_size * 1024, virtual_size * 1024)
 
+# XXX - commented as it's still not implemented on all platforms.
+# Leaving it here for the future.
+##    @prevent_zombie
+##    @wrap_privileges
+##    def get_process_environ(self, pid):
+##        """Return process environment variables for the process with the
+##        given PID as a dictionary."""
+##        if pid == 0:
+##            return {}
+##        f = open("/proc/%s/environ" % pid)
+##        envs = f.read().strip('\0').split('\0')
+##        f.close()
+##        dict = {}
+##        for env in envs:
+##            key, value = env.split('=', 1)
+##            dict[key] = value
+##        return dict
+
+    @prevent_zombie
+    @wrap_privileges
+    def get_process_cwd(self, pid):
+        if pid == 0:
+            return ''
+        return os.readlink("/proc/%s/cwd" %pid)
+
     def _get_ppid(self, pid):
         f = open("/proc/%s/status" % pid)
         for line in f:
