@@ -4,11 +4,17 @@
 #
 
 import errno
+import os
 import _psutil_mswindows
 
 # import psutil exceptions we can override with our own
 from error import *
 
+try:
+    import wmi
+except ImportError:
+    wmi = None
+
 
 # --- module level constants (gets pushed up to psutil module)
 
@@ -17,6 +23,8 @@ NUM_CPUS = _psutil_mswindows.get_num_cpus()
 _UPTIME = _psutil_mswindows.get_system_uptime()
 TOTAL_PHYMEM = _psutil_mswindows.get_total_phymem()
 
+ERROR_ACCESS_DENIED = 5
+ERROR_INVALID_PARAMETER = 87
 
 # --- public functions
 
@@ -57,7 +65,7 @@ def wrap_privileges(callable):
         try:
             return callable(*args, **kwargs)
         except OSError, err:
-            if err.errno == errno.EACCES:
+            if err.errno in (errno.EACCES, ERROR_ACCESS_DENIED):
                 raise AccessDenied
             raise
     return wrapper
@@ -88,10 +96,24 @@ class Impl(object):
             return _psutil_mswindows.kill_process(pid)
         except OSError, err:
             # work around issue #24
-            if (pid == 0) and (err.errno == errno.EINVAL):
+            if (pid == 0) and err.errno in (errno.EINVAL, ERROR_INVALID_PARAMETER):
                 raise AccessDenied
             raise
 
+    def get_process_username(self, pid):
+        """Return the name of the user that owns the process"""
+        if wmi is None:
+            raise NotImplementedError("This functionnality requires pywin32 " \
+                                      "extension to be installed")
+        else:
+            if pid in (0, 4):
+                return 'NT AUTHORITY\\SYSTEM'
+            w = wmi.WMI().Win32_Process(ProcessId=pid)
+            if not w:
+                raise NoSuchProcess
+            domain, _, username = w[0].GetOwner()
+            return "%s\\%s" %(domain, username)
+
     def get_pid_list(self):
         """Returns a list of PIDs currently running on the system."""
         return _psutil_mswindows.get_pid_list()
@@ -110,3 +132,17 @@ class Impl(object):
     def get_cpu_times(self, pid):
         return _psutil_mswindows.get_process_cpu_times(pid)
 
+    def suspend_process(self, pid):
+        return _psutil_mswindows.suspend_process(pid)
+
+    def resume_process(self, pid):
+        return _psutil_mswindows.resume_process(pid)
+
+    @wrap_privileges
+    def get_process_cwd(self, pid):
+        if pid in (0, 4):
+            return ''
+        # return a normalized pathname since the native C function appends
+        # "\\" at the and of the path
+        path = _psutil_mswindows.get_process_cwd(pid)
+        return os.path.normpath(path)
