@@ -20,6 +20,8 @@
 #include "arch/bsd/process_info.h"
 
 
+static PyObject *NoSuchProcessException;
+
 /*
  * define the psutil C module methods and initialize the module.
  */
@@ -53,18 +55,81 @@ static PyMethodDef PsutilMethods[] =
 };
 
 
-static PyObject *NoSuchProcessException;
+struct module_state {
+    PyObject *error;
+};
 
-PyMODINIT_FUNC
-init_psutil_bsd(void)
-{
-     PyObject *m;
-     m = Py_InitModule("_psutil_bsd", PsutilMethods);
-     NoSuchProcessException = PyErr_NewException("_psutil_bsd.NoSuchProcess", NULL, NULL);
-     Py_INCREF(NoSuchProcessException);
-     PyModule_AddObject(m, "NoSuchProcess", NoSuchProcessException);
+#if PY_MAJOR_VERSION >= 3
+#define GETSTATE(m) ((struct module_state*)PyModule_GetState(m))
+#else
+#define GETSTATE(m) (&_state)
+static struct module_state _state;
+#endif
+
+#if PY_MAJOR_VERSION >= 3
+
+static int psutil_bsd_traverse(PyObject *m, visitproc visit, void *arg) {
+    Py_VISIT(GETSTATE(m)->error);
+    return 0;
 }
 
+static int psutil_bsd_clear(PyObject *m) {
+    Py_CLEAR(GETSTATE(m)->error);
+    return 0;
+}
+
+
+static struct PyModuleDef moduledef = {
+        PyModuleDef_HEAD_INIT,
+        "psutil_bsd",
+        NULL,
+        sizeof(struct module_state),
+        PsutilMethods,
+        NULL,
+        psutil_bsd_traverse,
+        psutil_bsd_clear,
+        NULL
+};
+
+#define INITERROR return NULL
+
+PyObject *
+PyInit__psutil_bsd(void)
+
+#else
+#define INITERROR return
+
+void
+init_psutil_bsd(void)
+#endif
+{
+#if PY_MAJOR_VERSION >= 3
+    PyObject *module = PyModule_Create(&moduledef);
+#else
+    PyObject *module = Py_InitModule("_psutil_bsd", PsutilMethods);
+#endif
+    if (module == NULL)
+        INITERROR;
+    struct module_state *st = GETSTATE(module);
+
+    st->error = PyErr_NewException("_psutil_bsd.Error", NULL, NULL);
+    if (st->error == NULL) {
+        Py_DECREF(module);
+        INITERROR;
+    }
+
+    NoSuchProcessException = PyErr_NewException("_psutil_bsd.NoSuchProcess",
+                                                 NULL, NULL);
+    Py_INCREF(NoSuchProcessException);
+    PyModule_AddObject(module, "NoSuchProcess", NoSuchProcessException);
+
+#if PY_MAJOR_VERSION >= 3
+    return module;
+#endif
+}
+
+
+
 
 /*
  * Return a Python list of all the PIDs running on the system.
@@ -156,8 +221,9 @@ static PyObject* get_process_info(PyObject* self, PyObject* args)
 
         // hooray, we got all the data, so return it as a tuple to be passed to
         // ProcessInfo() constructor
-        infoTuple = Py_BuildValue("llssNll", pid, kp.ki_ppid, kp.ki_comm, "",
-                                  arglist, kp.ki_ruid, kp.ki_rgid);
+        infoTuple = Py_BuildValue("llssNll", pid, (long)kp.ki_ppid, kp.ki_comm, "",
+                                  arglist, (long)kp.ki_ruid, (long)kp.ki_rgid);
+
         if (NULL == infoTuple) {
             PyErr_SetString(PyExc_RuntimeError,
                             "Failed to build process information tuple!");
@@ -329,8 +395,8 @@ static PyObject* get_memory_info(PyObject* self, PyObject* args)
  */
 static PyObject* get_total_phymem(PyObject* self, PyObject* args)
 {
+    long total_phymem;
     int mib[2];
-    int total_phymem;
     size_t len;
 
     mib[0] = CTL_HW;
@@ -342,7 +408,7 @@ static PyObject* get_total_phymem(PyObject* self, PyObject* args)
         return NULL;
     }
 
-    return Py_BuildValue("i", total_phymem);
+    return Py_BuildValue("l", total_phymem);
 }
 
 
@@ -352,13 +418,13 @@ static PyObject* get_total_phymem(PyObject* self, PyObject* args)
  */
 static PyObject* get_avail_phymem(PyObject* self, PyObject* args)
 {
-    unsigned int v_inactive_count;
-    unsigned int v_cache_count;
-    unsigned int v_free_count;
-    int total_mem;
-    long long avail_mem;
-    size_t size = sizeof(unsigned int);
-    size_t psize = sizeof(int);
+    unsigned long v_inactive_count = 0;
+    unsigned long v_cache_count = 0;
+    unsigned long v_free_count = 0;
+    long total_mem = 0;
+    long long avail_mem = 0;
+    size_t size = sizeof(unsigned long);
+    size_t psize = sizeof(total_mem);
     int pagesize = getpagesize();
 
     if (sysctlbyname("hw.physmem", &total_mem, &psize, NULL, 0) == -1) {
