@@ -6,6 +6,7 @@
  */
 
 #include <Python.h>
+#include <windows.h>
 
 static PyObject* get_pid_list(PyObject* self, PyObject* args);
 static PyObject* kill_process(PyObject* self, PyObject* args);
@@ -21,5 +22,8 @@ static PyObject* get_total_virtmem(PyObject* self, PyObject* args);
 static PyObject* get_avail_phymem(PyObject* self, PyObject* args);
 static PyObject* get_avail_virtmem(PyObject* self, PyObject* args);
 static PyObject* get_system_cpu_times(PyObject* self, PyObject* args);
+static PyObject* get_process_cwd(PyObject* self, PyObject* args);
+static PyObject* suspend_process(PyObject* self, PyObject* args);
+static PyObject* resume_process(PyObject* self, PyObject* args);
 
-
+int suspend_resume_process(DWORD pid, int suspend);
