@@ -24,6 +24,9 @@
 #include "arch/osx/process_info.h"
 
 
+static PyObject *NoSuchProcessException;
+static PyObject *AccessDeniedException;
+
 /*
  * define the psutil C module methods and initialize the module.
  */
@@ -56,25 +59,87 @@ static PyMethodDef PsutilMethods[] =
 };
 
 
-static PyObject *NoSuchProcessException;
-static PyObject *AccessDeniedException;
 
-PyMODINIT_FUNC
+
+struct module_state {
+    PyObject *error;
+};
+
+#if PY_MAJOR_VERSION >= 3
+#define GETSTATE(m) ((struct module_state*)PyModule_GetState(m))
+#else
+#define GETSTATE(m) (&_state)
+static struct module_state _state;
+#endif
+
+#if PY_MAJOR_VERSION >= 3
+
+static int psutil_osx_traverse(PyObject *m, visitproc visit, void *arg) {
+    Py_VISIT(GETSTATE(m)->error);
+    return 0;
+}
+
+static int psutil_osx_clear(PyObject *m) {
+    Py_CLEAR(GETSTATE(m)->error);
+    return 0;
+}
+
+
+static struct PyModuleDef moduledef = {
+        PyModuleDef_HEAD_INIT,
+        "psutil_osx",
+        NULL,
+        sizeof(struct module_state),
+        PsutilMethods,
+        NULL,
+        psutil_osx_traverse,
+        psutil_osx_clear,
+        NULL
+};
+
+#define INITERROR return NULL
+
+PyObject *
+PyInit__psutil_osx(void)
+
+#else
+#define INITERROR return
+
+void
 init_psutil_osx(void)
+#endif
 {
-     PyObject *m;
-     m = Py_InitModule("_psutil_osx", PsutilMethods);
+#if PY_MAJOR_VERSION >= 3
+    PyObject *module = PyModule_Create(&moduledef);
+#else
+    PyObject *module = Py_InitModule("_psutil_osx", PsutilMethods);
+#endif
+    if (module == NULL)
+        INITERROR;
+    struct module_state *st = GETSTATE(module);
+
+    st->error = PyErr_NewException("_psutil_osx.Error", NULL, NULL);
+    if (st->error == NULL) {
+        Py_DECREF(module);
+        INITERROR;
+    }
+
      NoSuchProcessException = PyErr_NewException("_psutil_osx.NoSuchProcess",
                                                  NULL, NULL);
      Py_INCREF(NoSuchProcessException);
-     PyModule_AddObject(m, "NoSuchProcess", NoSuchProcessException);
+     PyModule_AddObject(module, "NoSuchProcess", NoSuchProcessException);
      AccessDeniedException = PyErr_NewException("_psutil_osx.AccessDenied",
                                                 NULL, NULL);
      Py_INCREF(AccessDeniedException);
-     PyModule_AddObject(m, "AccessDenied", AccessDeniedException);
+     PyModule_AddObject(module, "AccessDenied", AccessDeniedException);
+
+#if PY_MAJOR_VERSION >= 3
+    return module;
+#endif
 }
 
 
+
 /*
  * Return a Python list of all the PIDs running on the system.
  */
@@ -87,7 +152,7 @@ static PyObject* get_pid_list(PyObject* self, PyObject* args)
     PyObject *pid;
     PyObject *retlist = PyList_New(0);
 
-    if (GetBSDProcessList(&proclist, &num_processes) != 0) {
+    if (get_proc_list(&proclist, &num_processes) != 0) {
         Py_DECREF(retlist);
         PyErr_SetString(PyExc_RuntimeError, "failed to retrieve process list.");
         return NULL;
@@ -315,7 +380,6 @@ static PyObject* get_process_create_time(PyObject* self, PyObject* args)
                             "Invalid argument - no PID provided.");
 	}
 
-    len = 4;
     mib[0] = CTL_KERN;
     mib[1] = KERN_PROC;
     mib[2] = KERN_PROC_PID;
@@ -329,6 +393,15 @@ static PyObject* get_process_create_time(PyObject* self, PyObject* args)
         return Py_BuildValue("d", TV2DOUBLE(kp.kp_proc.p_starttime));
     }
 
+    /*
+     * if sysctl succeeds but len is zero, raise NoSuchProcess as this only
+     * appears to happen when the process has gone away.
+     */
+    else {
+        return PyErr_Format(NoSuchProcessException,
+                            "No process found with pid %lu", pid);
+    }
+
     return PyErr_Format(PyExc_RuntimeError, "Unable to read process start time.");
 }
 
@@ -344,6 +417,10 @@ static PyObject* get_memory_info(PyObject* self, PyObject* args)
     task_port_t task = (task_port_t)NULL;
     time_value_t user_time, system_time;
     struct task_basic_info tasks_info;
+    vm_region_basic_info_data_64_t  b_info;
+    vm_address_t address = GLOBAL_SHARED_TEXT_SEGMENT;
+    vm_size_t size;
+    mach_port_t object_name;
 
     // the argument passed should be a process id
 	if (! PyArg_ParseTuple(args, "l", &pid)) {
@@ -365,6 +442,21 @@ static PyObject* get_memory_info(PyObject* self, PyObject* args)
                 return PyErr_Format(PyExc_RuntimeError,
                             "task_info(TASK_BASIC_INFO) failed for pid %lu", pid);
         }
+
+        /* Issue #73 http://code.google.com/p/psutil/issues/detail?id=73
+         * adjust the virtual memory size down to account for
+         * shared memory that task_info.virtual_size includes w/every process
+         */
+        info_count = VM_REGION_BASIC_INFO_COUNT_64;
+        err = vm_region_64(task, &address, &size, VM_REGION_BASIC_INFO,
+            (vm_region_info_t)&b_info, &info_count, &object_name);
+        if (err == KERN_SUCCESS) {
+            if (b_info.reserved && size == (SHARED_TEXT_REGION_SIZE) &&
+                tasks_info.virtual_size > (SHARED_TEXT_REGION_SIZE + SHARED_DATA_REGION_SIZE)) {
+                    tasks_info.virtual_size -= (SHARED_TEXT_REGION_SIZE + SHARED_DATA_REGION_SIZE);
+            }
+        }
+
     }
 
     else {
