@@ -32,7 +32,7 @@ int get_proc_list(struct kinfo_proc **procList, size_t *procCount)
     int err;
     struct kinfo_proc * result;
     int done;
-    static const int name[] = { CTL_KERN, KERN_PROC, KERN_PROC_ALL, 0 };
+    static const int name[] = { CTL_KERN, KERN_PROC, KERN_PROC_PROC, 0 };
     // Declaring name as const requires us to cast it when passing it to
     // sysctl because the prototype doesn't include the const modifier.
     size_t              length;
@@ -140,6 +140,7 @@ char *getcmdpath(long pid, size_t *pathsize)
     *pathsize = size;
     if (sysctl(mib, 4, path, &size, NULL, 0) == -1) {
         //perror("sysctl");
+        free(path);
         return NULL;       /* Insufficient privileges */
     }
 
@@ -162,7 +163,7 @@ char *getcmdpath(long pid, size_t *pathsize)
  */
 char *getcmdargs(long pid, size_t *argsize)
 {
-    int mib[3];
+    int mib[4];
     size_t size, argmax;
     char *procargs = NULL;
 
