@@ -10,6 +10,7 @@
 #include <Psapi.h>
 #include <tlhelp32.h>
 
+#include "security.h"
 #include "process_info.h"
 
 
@@ -107,6 +108,39 @@ DWORD* get_pids(DWORD *numberOfReturnedPIDs){
 }
 
 
+int is_system_proc(DWORD pid) {
+    HANDLE hProcess;
+
+    // Special case for PID 0 System Idle Process
+    // and PID 4 (SYSTEM)
+    if ((pid == 0) || (pid == 4)) {
+        return 1;
+    }
+
+    if (pid < 0) {
+        return 0;
+    }
+
+    hProcess = handle_from_pid(pid);
+    if (NULL == hProcess) {
+        // invalid parameter is no such process
+        if (GetLastError() == ERROR_INVALID_PARAMETER) {
+            return 0;
+        }
+
+        // access denied obviously means there's a process to deny access to...
+        if (GetLastError() == ERROR_ACCESS_DENIED) {
+            return 1;
+        }
+
+        PyErr_SetFromWindowsErr(0);
+        return -1;
+    }
+
+    return HasSystemPrivilege(hProcess);
+}
+
+
 int pid_is_running(DWORD pid)
 {
     HANDLE hProcess;
@@ -334,15 +368,14 @@ PyObject* get_arg_list(long pid)
     if (NULL == szArglist) {
         // failed to parse arglist
         // encode as a UTF8 Python string object from WCHAR string
-        argList = Py_BuildValue("N",
-                                PyUnicode_AsUTF8String(
-                                    PyUnicode_FromWideChar(
-                                    commandLineContents, commandLine.Length / 2)
-                                    )
-                                );
+        arg_from_wchar = PyUnicode_FromWideChar(commandLineContents,
+                                                commandLine.Length / 2);
+        #if PY_MAJOR_VERSION >= 3
+            argList = Py_BuildValue("N", PyUnicode_AsUTF8String(arg_from_wchar));
+        #else
+            argList = Py_BuildValue("N", PyUnicode_FromObject(arg_from_wchar));
+        #endif
     }
-
-
     else {
         // arglist parsed as array of UNICODE_STRING, so convert each to Python
         // string object and add to arg list
@@ -353,7 +386,11 @@ PyObject* get_arg_list(long pid)
             arg_from_wchar = PyUnicode_FromWideChar(szArglist[i],
                                                     wcslen(szArglist[i])
                                                     );
-            arg = PyUnicode_AsUTF8String(arg_from_wchar);
+            #if PY_MAJOR_VERSION >= 3
+                arg = PyUnicode_FromObject(arg_from_wchar);
+            #else
+                arg = PyUnicode_AsUTF8String(arg_from_wchar);
+            #endif
             Py_XDECREF(arg_from_wchar);
             PyList_Append(argList, arg);
             Py_XDECREF(arg);
