@@ -5,6 +5,7 @@
 
 import os
 import gc
+import sys
 import unittest
 
 import psutil
@@ -56,6 +57,11 @@ class TestProcessObjectLeaks(unittest.TestCase):
         # includes name, ppid, path, cmdline, uid, gid properties
         self.execute('__str__')
 
+    # XXX - disabled for Windows as it uses WMI which takes ages to complete
+    if not sys.platform.lower().startswith("win32"):
+        def test_username(self):
+            self.execute('username')
+
     def test_create_time(self):
         self.execute('create_time')
 
@@ -74,6 +80,13 @@ class TestProcessObjectLeaks(unittest.TestCase):
     def test_is_running(self):
         self.execute('is_running')
 
+    def test_resume(self):
+        self.execute('resume')
+
+    if hasattr(psutil.Process, "getcwd"):
+        def test_getcwd(self):
+            self.execute('getcwd')
+
 
 class TestModuleFunctionsLeaks(unittest.TestCase):
     """Test leaks of psutil module functions."""
@@ -139,6 +152,7 @@ class TestModuleFunctionsLeaks(unittest.TestCase):
     def test_cpu_percent(self):
         self.execute('cpu_percent')
 
+
 def test_main():
     test_suite = unittest.TestSuite()
     test_suite.addTest(unittest.makeSuite(TestProcessObjectLeaks))
